/'  IupColorBrowser: Example in C 
    Creates a IupColorBrowser element and updates, via callbacks, the values of
    texts representing the R, G and B colors selected.
'/

#define NULL Cast(Any Ptr,0)
 
/' ANSI C libraries include '/
#include "crt/stdlib.bi"
#include "crt/stdio.bi"

/' IUP libraries include '/
#include "iup3/iup.bi"
#include "iup3/iupcontrols.bi"

/' Global IUP handles '/
Dim Shared As Ihandle Ptr text_red, text_green, text_blue
Dim Shared As Ihandle Ptr lbl_state

/'*****************************************************************************
 * Function:                                                                  *
 * ColorBrowser drag callback                                                 *
 *                                                                            *
 * Description:                                                               *
 * This callback is associated to an intermediate color change                *
 *                                                                            *
 * Parameters received:                                                       *
 * self - Element identifier                                                  *
 * r    - red color component                                                 * 
 * g    - green color component                                               *
 * b    - blue color component                                                *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function clrbrwsr_drag_cb cdecl (ByVal self As Ihandle Ptr, ByVal r As UByte , _
																	ByVal g As UByte, ByVal b As UByte) As Integer

  Dim As String * 10 buf

  buf = Str(r)
  IupSetAttribute(text_red, "VALUE", StrPtr(buf))
  buf = Str(g)
  IupSetAttribute(text_green, "VALUE", StrPtr(buf))
  buf = Str(b)
  IupSetAttribute(text_blue, "VALUE", StrPtr(buf))

  /' Gives IUP a time to call the OS and update the text field. '/
  IupLoopStep()

  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * ColorBrowser drag callback                                                 *
 *                                                                            *
 * Description:                                                               *
 * This callback is associated to a definitive color change                   *
 *                                                                            *
 * Parameters received:                                                       *
 * self - Element identifier                                                  *
 * r    - red color component                                                 * 
 * g    - green color component                                               *
 * b    - blue color component                                                *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function clrbrwsr_change_cb cdecl (ByVal self As Ihandle Ptr, ByVal r As UByte , _
																	ByVal g As UByte, ByVal b As UByte) As Integer

  clrbrwsr_drag_cb(self, r, g, b)
  
  Return IUP_DEFAULT

End Function

/' Main program '/
Function main As Integer

  /' IUP handles '/
  Dim As Ihandle Ptr dlg
  Dim As Ihandle Ptr hbox_final
  Dim As Ihandle Ptr vbox
  Dim As Ihandle Ptr clrbrwsr
  
  /' Initializes IUP '/
  IupOpen(NULL, NULL)
  
  /' Initializes IUP controls '/
  IupControlsOpen ()
  
  /' Program begin '/

  /' Creates three texts '/
  text_red = IupText (NULL)
  text_green = IupText (NULL)
  text_blue = IupText (NULL)

  /' Creates a color browser '/
  clrbrwsr = IupColorBrowser ()
  
  /' Registers color browser callbacks '/
  IupSetCallback(clrbrwsr, "CHANGE_CB", Cast(Icallback,@clrbrwsr_change_cb))
  IupSetCallback(clrbrwsr, "DRAG_CB", Cast(Icallback,@clrbrwsr_drag_cb))

  /' Creates a vbox '/
  vbox = IupVbox _
         ( _
           IupFill(), _
           text_red, _
           IupFill(), _
           text_green, _
           IupFill(), _
           text_blue, _
           IupFill(), _
           NULL _
         )

  /' Creates an hbox '/
  hbox_final = IupHbox (clrbrwsr, IupFill(), vbox, NULL)

  /' Creates a dialog '/
  dlg = IupDialog (hbox_final)
  
  /' Sets title of the dialog '/
  IupSetAttribute (dlg, "TITLE", "IupColorBrowser")
  
  /' Shows dialog in the center of the screen '/
  IupShowXY (dlg, IUP_CENTER, IUP_CENTER)
  
  /' Initializes IUP main loop '/
  IupMainLoop ()

  /' Finishes IUP '/
  IupClose ()	

  /' Program finished successfully '/
  Return EXIT_SUCCESS

End Function

main