/'* \file
 * \brief User API
 * IUP - A Portable User Interface Toolkit
 * Tecgraf: Computer Graphics Technology Group, PUC-Rio, Brazil
 * http://www.tecgraf.puc-rio.br/iup  mailto:iup@tecgraf.puc-rio.br
 *
 * See Copyright Notice at the end of this file
 '/
 
#ifndef __IUP_BI__
#define __IUP_BI__

#inclib "iup"
#include "iup3/iupkey.bi"
'#include "iup3/iupdef.bi"
#inclib "shell32"
#inclib "gdi32"
#inclib "comctl32"
#inclib "ole32"
#inclib "comdlg32"
#inclib "user32"
#inclib "advapi32.dll"


#define IUP_NAME "IUP - Portable User Interface"
#define IUP_COPYRIGHT  "Copyright (C) 1994-2009 Tecgraf, PUC-Rio."
#define IUP_DESCRIPTION	"Portable toolkit for building graphical user interfaces."
#define IUP_VERSION "3.0beta3"
#define IUP_VERSION_DATE "2009/04/04"
#define IUP_VERSION_NUMBER 300000

Type Ihandle As Any
Type Icallback As Function cdecl (ByVal As Ihandle Ptr,...) As Integer

Extern "C"



/'**********************************************************************'/
/'                      pre-definided dialogs                           '/
/'**********************************************************************'/
Declare Function IupFileDlg () As Ihandle Ptr
Declare Function IupMessageDlg () As Ihandle Ptr
Declare Function IupColorDlg () As Ihandle Ptr
Declare Function IupFontDlg () As Ihandle Ptr

Declare Function IupGetFile (ByVal arq As ZString Ptr) As Integer
Declare Sub IupMessage (ByVal title As Const ZString Ptr, ByVal msg As Const ZString Ptr)
Declare Sub IupMessagef (ByVal title As Const ZString Ptr, ByVal format As Const ZString Ptr, ...)
Declare Function IupAlarm (ByVal title As Const ZString Ptr, ByVal msg As Const ZString Ptr, _
													ByVal b1 As Const ZString Ptr, ByVal b2 As Const ZString Ptr, _
													ByVal b3 As Const ZString Ptr) As Integer
Declare Function IupScanf (ByVal format As Const ZString Ptr, ...) As Integer
Declare Function IupListDialog (ByVal type_ As Integer, ByVal title As Const ZString Ptr, _
																ByVal size As Integer, ByVal lst As Const ZString Ptr Ptr, _
																ByVal op As Integer, ByVal col As Integer, _
																ByVal line_ As Integer, ByVal marks As Integer Ptr) As Integer
Declare Function IupGetText (ByVal title As Const ZString Ptr, ByVal text As ZString Ptr) As Integer
Declare Function IupGetColor (ByVal x As Integer, ByVal y As Integer, ByVal r As UByte Ptr, _
															ByVal g As UByte Ptr, ByVal b As UByte Ptr) As Integer

Type Iparamcb As Function cdecl (ByVal dialog As Ihandle Ptr, ByVal param_index As Integer, _
													ByVal user_data As Any Ptr) As Integer

Declare Function IupGetParam (ByVal title As Const ZString Ptr, ByVal action As Iparamcb, _
														ByVal user_data As Any Ptr, ByVal format_ As Const ZString Ptr,...) As Integer
Declare Function IupGetParamv (ByVal title As Const ZString Ptr, ByVal action As Iparamcb , _
															ByVal user_data As Any Ptr, ByVal format As Const ZString Ptr, _
															ByVal param_count As Integer, ByVal param_extra As Integer, _
															ByVal param_data As Any Ptr Ptr) As Integer


/'**********************************************************************'/
/'                      Functions                                       '/
/'**********************************************************************'/
'/
Declare Function IupOpen 					(ByVal argc As Integer Ptr, Byval argv As ZString Ptr Ptr Ptr) As Integer
Declare Sub IupClose          ()
Declare Sub IupImageLibOpen   ()

Declare Function IupMainLoop       () As Integer
Declare Function IupLoopStep       () As Integer
Declare Function IupMainLoopLevel  () As Integer
Declare Sub IupFlush          ()
Declare Sub IupExitLoop       ()

Declare Sub IupUpdate 	(ByVal ih As Ihandle Ptr)
Declare Sub IupUpdateChildren (ByVal ih As Ihandle Ptr)
Declare Sub IupRedraw 	(ByVal ih As Ihandle Ptr, ByVal children As Integer)
Declare Sub IupRefresh 	(ByVal ih As Ihandle Ptr)

Declare Function IupMapFont 	(ByVal iupfont As Const ZString Ptr) As ZString Ptr
Declare Function IupUnMapFont      (ByVal driverfont As Const ZString Ptr) As ZString Ptr
Declare Function IupHelp           (ByVal url As Const ZString Ptr) As Integer
Declare Function IupLoad           (ByVal filename As Const ZString Ptr) As ZString Ptr

Declare Function IupVersion        () As ZString Ptr
Declare Function IupVersionDate    () As ZString Ptr
Declare Function IupVersionNumber  () As Integer
Declare Sub IupSetLanguage    (ByVal lng As Const ZString Ptr)
Declare Function IupGetLanguage    () As ZString Ptr

Declare Sub IupDestroy       (ByVal ih As Ihandle Ptr)
Declare Sub IupDetach        (ByVal child As Ihandle Ptr)
Declare Function IupAppend        (ByVal ih As Ihandle Ptr, ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupInsert        (ByVal ih As Ihandle Ptr, ByVal ref_child As Ihandle Ptr, _
																		ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetChild      (ByVal ih As Ihandle Ptr, ByVal pos_ As Integer) As Ihandle Ptr
Declare Function IupGetChildPos   (ByVal ih As Ihandle Ptr, ByVal child As Ihandle Ptr) As Integer
Declare Function IupGetChildCount (ByVal ih As Ihandle Ptr) As Integer
Declare Function IupGetNextChild  (ByVal ih As Ihandle Ptr, ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetBrother    (ByVal ih As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetParent     (ByVal ih As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetDialog     (ByVal ih As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetDialogChild (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As Ihandle Ptr
Declare Function IupReparent      (ByVal ih As Ihandle Ptr, ByVal new_parent As Ihandle Ptr) As Integer

Declare Function IupPopup          (ByVal ih As Ihandle Ptr, ByVal x As Integer, ByVal y As Integer) As Integer
Declare Function IupShow           (ByVal ih As Ihandle Ptr) As Integer
Declare Function IupShowXY         (ByVal ih As Ihandle Ptr, ByVal x As Integer, ByVal y As Integer) As Integer
Declare Function IupHide           (ByVal ih As Ihandle Ptr) As Integer
Declare Function IupMap            (ByVal ih As Ihandle Ptr) As Integer
Declare Sub IupUnmap          (ByVal ih As Ihandle Ptr)

Declare Sub IupSetAttribute   (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr, _
																	ByVal value As Const ZString Ptr)
Declare Sub IupStoreAttribute (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr, _
																	ByVal value As Const ZString Ptr)
Declare Function IupSetAttributes  (ByVal ih As Ihandle Ptr, ByVal str As Const ZString Ptr) As Ihandle Ptr
Declare Function IupGetAttribute   (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As ZString Ptr
Declare Function IupGetAttributes  (ByVal ih As Ihandle Ptr) As ZString Ptr
Declare Function IupGetInt         (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As Integer
Declare Function IupGetInt2        (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As Integer
Declare Function IupGetIntInt      (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr, _
																		ByVal i1 As Integer Ptr, ByVal i2 As Integer Ptr) As Integer
Declare Function IupGetFloat       (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As Single
Declare Sub IupSetfAttribute  (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr, _
																		ByVal format As Const ZString Ptr, ...)
Declare Function IupGetAllAttributes (ByVal ih As Ihandle Ptr, ByVal names As ZString Ptr Ptr, Byval n As Integer) As Integer

Declare Sub IupSetGlobal      (ByVal name_ As Const ZString Ptr, ByVal value As Const ZString Ptr)
Declare Sub IupStoreGlobal    (ByVal name_ As Const ZString Ptr, ByVal value As Const ZString Ptr)
Declare Function IupGetGlobal      (ByVal name As Const ZString Ptr) As ZString Ptr

Declare Function IupSetFocus       (ByVal ih As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetFocus       () As Ihandle Ptr
Declare Function IupPreviousField  (ByVal ih As Ihandle Ptr) As Ihandle Ptr  
Declare Function IupNextField      (ByVal ih As Ihandle Ptr) As Ihandle Ptr

Declare Function IupGetCallback (ByVal ih As Ihandle Ptr, ByVal name_ As Const ZString Ptr) As Icallback
Declare Function IupSetCallback (ByVal ih As Ihandle Ptr, ByVal name_ As Const ZString Ptr, _
								ByVal func As Icallback) As Icallback
Declare Function IupSetCallbacks (ByVal ih As Ihandle Ptr, ByVal name_ As Const ZString Ptr, _
									ByVal func As Icallback, ...) As Ihandle Ptr

Declare Function IupGetFunction    (ByVal name As Const ZString Ptr) As Icallback
Declare Function IupSetFunction    (ByVal name As Const ZString Ptr, ByVal func As Icallback) As Icallback
Declare Function IupGetActionName  () As Const ZString Ptr

Declare Function IupGetHandle      (ByVal name As Const ZString Ptr) As Ihandle Ptr
Declare Function IupSetHandle      (ByVal name As Const ZString Ptr, ByVal ih As Ihandle Ptr) As Ihandle Ptr
Declare Function IupGetAllNames    (ByVal names As ZString Ptr Ptr, ByVal n As Integer) As Integer
Declare Function IupGetAllDialogs  (ByVal names As ZString Ptr Ptr, ByVal n As Integer) As Integer
Declare Function IupGetName        (ByVal ih As Ihandle Ptr) As ZString Ptr

Declare Sub IupSetAttributeHandle (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr, _
																			ByVal ih_named As Ihandle Ptr)
Declare Function IupGetAttributeHandle (ByVal ih As Ihandle Ptr, ByVal name As Const ZString Ptr) As Ihandle Ptr

Declare Function IupGetClassName (ByVal ih As Ihandle Ptr) As ZString Ptr
Declare Function IupGetClassType (ByVal ih As Ihandle Ptr) As ZString Ptr
Declare Function IupGetClassAttributes (ByVal classname As Const ZString Ptr, ByVal names As ZString Ptr Ptr, _
											ByVal n As Integer) As Integer
Declare Sub IupSaveClassAttributes (ByVal ih As Ihandle Ptr)
Declare Sub IupSetClassDefaultAttribute (ByVal classname As Const ZString Ptr, ByVal name As Const ZString Ptr, _
											ByVal value As Const ZString Ptr)

Declare Function IupCreate  (ByVal classname As Const ZString Ptr) As Ihandle Ptr
Declare Function IupCreatev (ByVal classname As Const ZString Ptr, ByVal params As Any Ptr Ptr) As Ihandle Ptr
Declare Function IupCreatep (ByVal classname As Const ZString Ptr, ByVal first As Any Ptr, ...) As Ihandle Ptr

Declare Function IupFill        () As Ihandle Ptr
Declare Function IupRadio       (ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupVbox        (ByVal child As Ihandle Ptr, ...) As Ihandle Ptr
Declare Function IupVboxv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr
Declare Function IupZbox        (ByVal child As Ihandle Ptr, ...) As Ihandle Ptr
Declare Function IupZboxv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr
Declare Function IupHbox        (ByVal child As Ihandle Ptr,...) As Ihandle Ptr
Declare Function IupHboxv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr

Declare Function IupNormalizer  (ByVal ih_first As Ihandle Ptr, ...) As Ihandle Ptr
Declare Function IupNormalizerv (ByVal ih_list As Ihandle Ptr Ptr) As Ihandle Ptr

Declare Function IupCbox        (ByVal child As Ihandle Ptr, ...) As Ihandle Ptr
Declare Function IupCboxv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr
Declare Function IupSbox        (Byval child As Ihandle Ptr) As Ihandle Ptr

Declare Function IupFrame       (ByVal child As Ihandle Ptr) As Ihandle Ptr

Declare Function IupImage       (ByVal width As Integer, ByVal height As Integer, _
																ByVal pixmap As Const UByte Ptr) As Ihandle Ptr
Declare Function IupImageRGB    (ByVal width As Integer, ByVal height As Integer, _
																ByVal pixmap As Const UByte Ptr) As Ihandle Ptr
Declare Function IupImageRGBA   (ByVal width As Integer, ByVal height As Integer, _
																	ByVal pixmap As Const UByte Ptr) As Ihandle Ptr

Declare Function IupItem        (ByVal title As Const ZString Ptr, ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupSubmenu     (ByVal title As Const ZString Ptr, ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupSeparator   () As Ihandle Ptr
Declare Function IupMenu        (ByVal child As Ihandle Ptr,...) As Ihandle Ptr
Declare Function IupMenuv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr

Declare Function IupButton      (ByVal title As Const ZString Ptr, ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupCanvas      (ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupDialog      (ByVal child As Ihandle Ptr) As Ihandle Ptr
Declare Function IupUser        () As Ihandle Ptr
Declare Function IupLabel       (ByVal title As Const ZString Ptr) As Ihandle Ptr
Declare Function IupList        (ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupText        (ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupMultiLine   (ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupToggle      (ByVal title As Const ZString Ptr, ByVal action As Const ZString Ptr) As Ihandle Ptr
Declare Function IupTimer       () As Ihandle Ptr

Declare Function IupSpin        () As Ihandle Ptr
Declare Function IupSpinbox     (ByVal child As Ihandle Ptr) As Ihandle Ptr

Declare Function IupProgressBar () As Ihandle Ptr
Declare Function IupVal         (ByVal type As Const ZString Ptr) As Ihandle Ptr
Declare Function IupTabs        (ByVal child As Ihandle Ptr, ...) As Ihandle Ptr
Declare Function IupTabsv       (ByVal children As Ihandle Ptr Ptr) As Ihandle Ptr

Declare Sub IupTextConvertXYToChar (ByVal ih As Ihandle Ptr, ByVal x As Integer, _
																				ByVal y As Integer, ByVal lin As Integer Ptr, _
																				ByVal col As Integer Ptr, ByVal pos_ As Integer Ptr)
Declare Sub IupListConvertXYToItem (ByVal ih As Ihandle Ptr, ByVal x As Integer, _
																				ByVal y As Integer, ByVal pos_ As Integer Ptr)


End Extern

/'**********************************************************************'/
/'                   Common Return Values                               '/
/'**********************************************************************'/
#define IUP_ERROR     1
#define IUP_NOERROR   0
#define IUP_OPENED   -1
#define IUP_INVALID  -1

/'**********************************************************************'/
/'                   Callback Return Values                             '/
/'**********************************************************************'/
#define IUP_IGNORE    -1
#define IUP_DEFAULT   -2
#define IUP_CLOSE     -3
#define IUP_CONTINUE  -4

/'**********************************************************************'/
/'           IupPopup and IupShowXY Parameter Values                    '/
/'**********************************************************************'/
#define IUP_CENTER        &hFFFF  /' 65535 '/
#define IUP_LEFT          &hFFFE  /' 65534 '/
#define IUP_RIGHT         &hFFFD  /' 65533 '/
#define IUP_MOUSEPOS      &hFFFC  /' 65532 '/
#define IUP_CURRENT       &hFFFB  /' 65531 '/
#define IUP_CENTERPARENT  &hFFFA  /' 65530 '/
#define IUP_TOP       IUP_LEFT
#define IUP_BOTTOM    IUP_RIGHT

/'**********************************************************************'/
/'               SHOW_CB Callback Values                                '/
/'**********************************************************************'/
Enum 
	IUP_SHOW
	IUP_RESTORE 
	IUP_MINIMIZE 
	IUP_MAXIMIZE
	IUP_HIDE
End Enum

/'**********************************************************************'/
/'               SCROLL_CB Callback Values                              '/
/'**********************************************************************'/
Enum
	IUP_SBUP
	IUP_SBDN
	IUP_SBPGUP
	IUP_SBPGDN
	IUP_SBPOSV
	IUP_SBDRAGV
    IUP_SBLEFT
    IUP_SBRIGHT
    IUP_SBPGLEFT
    IUP_SBPGRIGHT
    IUP_SBPOSH
    IUP_SBDRAGH
End Enum

/'**********************************************************************'/
/'               Mouse Button Values and Macros                         '/
/'**********************************************************************'/
#define IUP_BUTTON1   Asc("1")
#define IUP_BUTTON2   Asc("2")
#define IUP_BUTTON3   Asc("3")
#define IUP_BUTTON4   Asc("4")
#define IUP_BUTTON5   Asc("5")

#define iup_isshift(_s)    (_s[0]= Asc("S")
#define iup_iscontrol(_s)  (_s[1]= Asc("C")
#define iup_isbutton1(_s)  (_s[2]=Asc("1")
#define iup_isbutton2(_s)  (_s[3]=Asc("2")
#define iup_isbutton3(_s)  (_s[4]=Asc("3"))
#define iup_isdouble(_s)   (_s[5]=Asc("D")
#define iup_isalt(_s)      (_s[6]=Asc("A")
#define iup_issys(_s)      (_s[7]=Asc("Y")
#define iup_isbutton4(_s)  (_s[8]=Asc("4")
#define iup_isbutton5(_s)  (_s[9]=Asc("5")
#define isshift     iup_isshift
#define iscontrol   iup_iscontrol
#define isbutton1   iup_isbutton1
#define isbutton2   iup_isbutton2
#define isbutton3   iup_isbutton3
#define isdouble    iup_isdouble
#define isalt       iup_isalt
#define issys       iup_issys
#define isbutton4   iup_isbutton4
#define isbutton5   iup_isbutton5


/'**********************************************************************'/
/'                      Pre-Defined Masks                               '/
/'**********************************************************************'/
#define IUP_MASK_FLOAT    "[+/-]?(/d+/.?/d*|/./d+)"
#define IUP_MASK_UFLOAT   "(/d+/.?/d*|/./d+)"
#define IUP_MASK_EFLOAT   "[+/-]?(/d+/.?/d*|/./d+)([eE][+/-]?/d+)?"
#define IUP_MASK_INT      "[+/-]?/d+"
#define IUP_MASK_UINT     "/d+"
#define IUPMASK_FLOAT     IUP_MASK_FLOAT
#define IUPMASK_UFLOAT    IUP_MASK_UFLOAT
#define IUPMASK_EFLOAT    IUP_MASK_EFLOAT
#define IUPMASK_INT	      IUP_MASK_INT
#define IUPMASK_UINT      IUP_MASK_UINT


/' 
These definitions not needed when using IUP in combination with FreeBASIC

/'**********************************************************************'/
/'              Replacement for the WinMain in Windows,                 '/
/'        this allows the application to start from "main".             '/
/'**********************************************************************'/
#ifdef (__WATCOMC__)
Extern "C"
Declare Function IupMain (ByVal argc As Integer, ByVal argv As ZString Ptr Ptr) As Integer /' In C++ we have to Dim prototype As the cdecl  '/
End Extern
#endif
#define main IupMain /' this is the trick for Watcom and MetroWerks '/

'/

/'*****************************************************************************
* Copyright (C) 1994-2008 Tecgraf, PUC-Rio.
*
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
*
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*****************************************************************************'/
'/
#endif
