/'IupMultiline Advanced Example in C 
Shows a dialog with a multiline, a text, a list and some buttons. You can test the multiline attributes by clicking on the buttons. Each button is related to an attribute. Select if you want to set or get an attribute using the dropdown list. The value in the text will be used as value when a button is pressed. '/

#define NULL Cast(Any Ptr,0)
/' IupMultiLine example '/

/' ANSI C libraries include '/
#include "crt/stdlib.bi"
#include "crt/stdio.bi"

/' IUP libraries include '/
#include "iup3/iup.bi"

/'*****************************************************************************
 * Function:                                                                  *
 * Set Attribute                                                              *
 *                                                                            *
 * Description:                                                               *
 * Sets an attribute with the value in the text                               *
 *                                                                            *
 * Parameters received:                                                       *
 * attribute - attribute to be set                                            *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  set_attribute (ByVal attribute As ZString Ptr) As Integer

  Dim string_message As String * 50
  Dim As Ihandle Ptr mltline
  Dim As Ihandle Ptr text

  mltline = IupGetHandle ("mltline")
  text = IupGetHandle ("text")
  IupSetAttribute (mltline, attribute, IupGetAttribute (text, "VALUE"))

  sprintf (StrPtr(string_message), "Attribute %s set with value %s", attribute, IupGetAttribute (text, "VALUE"))

  IupMessage ("Set attribute", StrPtr(string_message))

  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Get attribute                                                              *
 *                                                                            *
 * Description:                                                               *
 * Get an attribute of the multiline and shows it in the text                 *
 *                                                                            *
 * Parameters received:                                                       *
 * attribute - attribute to be get                                            *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  get_attribute (Byval attribute As ZString Ptr) As Integer

  Dim As String * 50 string_message
  Dim As Ihandle Ptr mltline
  Dim As Ihandle Ptr text

  mltline = IupGetHandle ("mltline")
  text = IupGetHandle ("text")
  IupSetAttribute (text, "VALUE", IupGetAttribute(mltline, attribute))

  sprintf (StrPtr(string_message), "Attribute %s get with value %s", attribute, IupGetAttribute (text, "VALUE"))

  IupMessage ("Get attribute", StrPtr(string_message))

  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Functions:                                                                 *
 * Append button callback                                                     *
 *                                                                            *
 * Description:                                                               *
 * Appends text to the multiline. Value: text to be appended                  *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_append_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("APPEND")
	Else 
		get_attribute ("APPEND")
	End If

  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Insert button callback                                                     *
 *                                                                            *
 * Description:                                                               *
 * Inserts text in the multiline. Value: text to be inserted                  *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_insert_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("INSERT")
	Else 
		get_attribute ("INSERT")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Border button callback                                                     *
 *                                                                            *
 * Description:                                                               *
 * Border of the multiline. Value: "YES" or "NO"                              *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_border_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("BORDER")
	Else 
		get_attribute ("BORDER")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Caret button callback                                                      *
 *                                                                            *
 * Description:                                                               *
 * Position of the caret. Value: lin,col                                      *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_caret_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("CARET")
	Else 
		get_attribute ("CARET")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Read-only button callback                                                  *
 *                                                                            *
 * Description:                                                               *
 * Readonly attribute. Value: "YES" or "NO"                                   *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_readonly_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("READONLY")
	Else 
		get_attribute ("READONLY")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Selection button callback                                                  *
 *                                                                            *
 * Description:                                                               *
 * Changes the selection attribute. Value: lin1,col1:lin2,col2                *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_selection_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("SELECTION")
	Else 
		get_attribute ("SELECTION")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Selected text button callback                                              *
 *                                                                            *
 * Description:                                                               *
 * Changes the selected text attribute. Value: lin1,col1:lin2,col2            *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_selectedtext_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("SELECTEDTEXT")
	Else 
		get_attribute ("SELECTEDTEXT")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Number of characters button callback                                       *
 *                                                                            *
 * Description:                                                               *
 * Limit number of characters in the multiline                                *
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_nc_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("NC")
	Else 
		get_attribute ("NC")
	End If
  
  Return IUP_DEFAULT
End Function

/'*****************************************************************************
 * Function:                                                                  *
 * Vaue button callback                                                       *
 *                                                                            *
 * Description:                                                               *
 * Text in the multiline.
 *                                                                            *
 * Value returned:                                                            *
 * IUP_DEFAULT                                                                *
 *****************************************************************************'/
Function  btn_value_cb cdecl () As Integer

  Dim As Ihandle Ptr list

  list = IupGetHandle ("list")

  If (IupGetInt (list, "VALUE") = 1) Then
		set_attribute ("VALUE")
	Else 
		get_attribute ("VALUE")
	End If
  
  Return IUP_DEFAULT
End Function

/' Main program '/
Sub main()

 /' IUP identifiers '/
  Dim As Ihandle Ptr dlg
  Dim As Ihandle Ptr mltline
  Dim As Ihandle Ptr text
  Dim As Ihandle Ptr list
  Dim As Ihandle Ptr btn_append, btn_insert, btn_border, btn_caret, btn_readonly, btn_selection, btn_selectedtext, btn_nc, btn_value

  /' Initializes IUP '/
  IupOpen(NULL,NULL)

  /' Program begin '/
 
  /' Creates a multiline, a text and a list'/ 
  mltline = IupMultiLine (NULL)
  text = IupText (NULL)
  list = IupList (NULL)

  /' Turns on multiline expand ans text horizontal expand '/
  IupSetAttribute (mltline, "EXPAND", "YES")
  IupSetAttribute (text, "EXPAND", "HORIZONTAL")

  /' Associates handles to multiline, text and list '/
  IupSetHandle ("mltline", mltline)
  IupSetHandle ("text", text)
  IupSetHandle ("list", list)

  /' Sets list items and dropdown '/
  IupSetAttributes (list, "1 = SET, 2 = GET, DROPDOWN = YES")

  /' Creates buttons '/
  btn_append = IupButton ("Append", NULL)
  btn_insert = IupButton ("Insert", NULL)
  btn_border = IupButton ("Border", NULL)
  btn_caret = IupButton ("Caret", NULL)
  btn_readonly = IupButton ("Read only", NULL)
  btn_selection = IupButton ("Selection", NULL)
  btn_selectedtext = IupButton ("Selected Text", NULL)
  btn_nc = IupButton ("Number of characters", NULL)
  btn_value = IupButton ("Value", NULL)

  /' Registers callbacks '/
  IupSetCallback(btn_append, "ACTION", Cast(Icallback,@btn_append_cb))
  IupSetCallback(btn_insert, "ACTION", Cast(Icallback,@btn_insert_cb))
  IupSetCallback(btn_border, "ACTION", Cast(Icallback,@btn_border_cb))
  IupSetCallback(btn_caret, "ACTION", Cast(Icallback,@btn_caret_cb))
  IupSetCallback(btn_readonly, "ACTION", Cast(Icallback,@btn_readonly_cb))
  IupSetCallback(btn_selection, "ACTION", Cast(Icallback,@btn_selection_cb))
  IupSetCallback(btn_selectedtext, "ACTION", Cast(Icallback,@btn_selectedtext_cb))
  IupSetCallback(btn_nc, "ACTION", Cast(Icallback,@btn_nc_cb))
  IupSetCallback(btn_value, "ACTION", Cast(Icallback,@btn_value_cb))
	
  /' Creates dialog '/
  dlg = IupDialog (IupVbox (mltline, _
                   IupHbox (text, list, NULL), _
                   IupHbox (btn_append, btn_insert, btn_border, btn_caret, btn_readonly, btn_selection, NULL), _
                   IupHbox (btn_selectedtext, btn_nc, btn_value, NULL),  _
                   NULL))

  /' Sets title and size of the dialog '/
  
	IupSetAttribute (dlg, "TITLE","IupMultiLine Example")
	IupSetAttribute (dlg, "SIZE","HALFxQUARTER")
  /' Shows dialog in the center of the screen '/
  IupShowXY (dlg, IUP_CENTER, IUP_CENTER)

  /' Initializes IUP main loop '/
  IupMainLoop ()

  /' Finishes IUP '/
  IupClose ()
	

  /' Program finished successfully '/
  

End Sub

main()