/' IupTree: Example in C 
   Creates a tree with some branches and leaves. 
   Two callbacks are registered: one deletes marked nodes when the Del key is pressed, 
   and the other, called when the right mouse button is pressed, opens a menu with options.
'/

#define NULL Cast(Any Ptr,0)

#include "crt/stdio.bi"
#include "crt/stdlib.bi"
#include "crt/string.bi"

#include "iup3/iup.bi"
#include "iup3/iupcontrols.bi"
#include "iup3/iupkey.bi"


/' Callback called when a leaf is added by the menu. '/
Function addleaf cdecl () As Integer

  Dim As String attr
  Dim As Ihandle Ptr tree_ = IupGetHandle("tree")
  Dim As Integer id = IupGetInt(tree_,"VALUE")
  attr = "ADDLEAF" & id	
  IupSetAttribute(tree_,StrPtr(attr),"")
  IupSetAttribute(tree_, "REDRAW", "YES")
  Return IUP_DEFAULT

End Function

/' Callback called when a branch is added by the menu. '/
Function addbranch cdecl () As Integer

  Dim As String attr
  Dim As Ihandle Ptr tree_ = IupGetHandle("tree")
  Dim As Integer id = IupGetInt(tree_,"VALUE")
  attr = "ADDBRANCH" & id	
  IupSetAttribute(tree_,StrPtr(attr),"")
  IupSetAttribute(tree_, "REDRAW", "YES")
  Return IUP_DEFAULT

End Function

/' Callback called when a node is removed by the menu. '/
Function removenode cdecl () As Integer

  Dim As Ihandle Ptr tree = IupGetHandle("tree")
  IupSetAttribute(tree,"DELNODE","MARKED")
  IupSetAttribute(tree, "REDRAW", "YES")
  Return IUP_DEFAULT

End Function

/' from the menu '/
Function renamenode cdecl () As Integer

	Return IUP_DEFAULT

End Function

Function executeleaf_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer) As Integer

  printf("executeleaf_cb (%d)\n", id)
  Return IUP_DEFAULT

End Function

Function renamenode_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer, ByVal name_ As ZString Ptr) As Integer

  printf("renamenode_cb (%d=%s)\n", id, name_)
  return IUP_DEFAULT

End Function

Function rename_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer, ByVal name_ As ZString Ptr) As Integer

  printf("rename_cb (%d=%s)\n", id, name_)
  If (strcmp(name_, "rename") = 0) Then
    Return IUP_IGNORE
	End If
  Return IUP_DEFAULT

End Function

Function branchopen_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer) As Integer

  printf("branchopen_cb (%d)\n", id)
  return IUP_DEFAULT

End Function

Function branchclose_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer) As Integer

  printf("branchclose_cb (%d)\n", id)
  Return IUP_DEFAULT

End Function

/'Function dragdrop_cb(ByVal h As Ihandle Ptr, ByVal drag_id As Integer, _
										Byval drop_id As Integer, ByVal isshift_ As Integer,_
										Byval iscontrol As Integer) As Integer

  printf("dragdrop_cb (%d)->(%d)\n", drag_id, drop_id)
  Return IUP_DEFAULT

End Function
'/

/' Callback called when a key is hit '/
Function k_any_cb cdecl (ByVal h As Ihandle Ptr, ByVal c As Integer) As Integer

  If (c = K_DEL) Then  
    IupSetAttribute(h,"DELNODE","MARKED")
    IupSetAttribute(h, "REDRAW", "YES")
  End If

  Return IUP_DEFAULT

End Function

Function selectnode cdecl (ByVal h As Ihandle Ptr) As Integer

  Dim As Ihandle Ptr tree = IupGetHandle("tree")
  IupSetAttribute(tree,"VALUE",IupGetAttribute(h, "TITLE"))  	
	IupSetAttribute(tree, "REDRAW", "YES")	
  Return IUP_DEFAULT

End Function


Function selection_cb (ByVal h As Ihandle Ptr, ByVal id As Integer, ByVal status As Integer) As Integer

	Dim As Ihandle Ptr tree = IupGetHandle("tree")  
	Dim As String attrib_value = "COLOR" & id
  'node was unselected, change textcolor back to black 
	
	If (status = 0) Then		
		IupSetAttribute(tree,StrPtr(attrib_value),"0 0 0")		
	'node was selected, change textcolor to white
	Else
		IupSetAttribute(tree,StrPtr(attrib_value),"255 255 255")		
	End If	
	IupSetAttribute(tree, "REDRAW", "YES")  
	Return IUP_DEFAULT

End Function

Function multiselect_cb cdecl (Byval h As Ihandle Ptr, ByVal ids As Integer Ptr, ByVal n As Integer) As Integer

	
	Dim As String attrib = "COLOR"
	If (n > 0) Then
		For i As Integer = 0 To n - 1
			attrib = attrib & ids[i]						
			IupSetAttribute(h,StrPtr(attrib),"0 0 0")      
		Next i		
	End If
	IupSetAttribute(h, "REDRAW", "YES")
	
	Return IUP_DEFAULT
	
End Function

/' Callback called when the right mouse button is pressed '/
Function rightclick_cb cdecl (ByVal h As Ihandle Ptr, Byval id As Integer) As Integer

  Dim As Ihandle Ptr popup_menu

  Dim As String id_string

  
	popup_menu = IupMenu( _
    IupItem ("Add Leaf","addleaf"), _
    IupItem ("Add Branch","addbranch"), _
    IupItem ("Rename Node","renamenode"), _
    IupItem ("Remove Node","removenode"), _
    IupSubmenu("Selection", IupMenu( _
      IupItem ("ROOT", "selectnode"), _
      IupItem ("LAST", "selectnode"), _
      IupItem ("PGUP", "selectnode"), _
      IupItem ("PGDN", "selectnode"), _
      IupItem ("NEXT", "selectnode"), _
      IupItem ("PREVIOUS", "selectnode"), _
      IupSeparator(), _
      IupItem ("INVERT", "selectnode"), _
      IupItem ("BLOCK", "selectnode"), _
      IupItem ("CLEARALL", "selectnode"), _
      IupItem ("MARKALL", "selectnode"), _
      IupItem ("INVERTALL", "selectnode"), _
      NULL)), _
    NULL)
    
  IupSetFunction("selectnode", Cast(Icallback,@selectnode))
  
  IupSetFunction("addleaf", Cast(Icallback,@addleaf))
  IupSetFunction("addbranch", Cast(Icallback,@addbranch))
  IupSetFunction("removenode", Cast(Icallback,@removenode))
  IupSetFunction("renamenode", Cast(Icallback,@renamenode))

  id_string = Str(id)
	IupSetAttribute(h, "VALUE", StrPtr(id_string))
  IupPopup(popup_menu,IUP_MOUSEPOS,IUP_MOUSEPOS)

  IupDestroy(popup_menu)

  Return IUP_DEFAULT

End Function

/' Initializes IupTree and registers callbacks '/
Sub init_tree()

  Dim As Ihandle Ptr tree_ = IupTree() 

  IupSetCallback(tree_, "EXECUTELEAF_CB", Cast(Icallback, @executeleaf_cb))
  IupSetCallback(tree_, "RENAMENODE_CB", Cast(Icallback,@renamenode_cb))
  IupSetCallback(tree_, "RENAME_CB", Cast(Icallback,@rename_cb))
  IupSetCallback(tree_, "BRANCHCLOSE_CB", Cast(Icallback,@branchclose_cb))
  IupSetCallback(tree_, "BRANCHOPEN_CB", Cast(Icallback,@branchopen_cb))
  'IupSetCallback(tree_, "DRAGDROP_CB", Cast(Icallback,@dragdrop_cb))
  IupSetCallback(tree_, "RIGHTCLICK_CB", Cast(Icallback,@rightclick_cb))
  IupSetCallback(tree_, "K_ANY", Cast(Icallback,@k_any_cb))
	IupSetCallback(tree_, "SELECTION_CB",Cast(Icallback,@selection_cb))
	IupSetCallback(tree_, "MULTISELECTION_CB",Cast(Icallback, @multiselect_cb))
	
	

  IupSetHandle("tree",tree_)

End Sub

/' Initializes the dialog '/
Sub init_dlg()

  Dim As Ihandle Ptr tree_ = IupGetHandle("tree")
  Dim As Ihandle Ptr box = IupVbox(IupHbox(tree_, IupButton("test", NULL), NULL), NULL)
  Dim As Ihandle Ptr dlg = IupDialog(box)
  IupSetAttribute(dlg, "TITLE", "IupTree")
	IupSetAttribute(dlg, "PLACEMENT", "MAXIMIZED")  
  IupSetAttribute(box, "MARGIN", "20x20")  	
  IupSetHandle("dlg",dlg)

End Sub

/' Initializes the IupTrees attributes '/
Sub init_tree_atributes()

  Dim As Ihandle Ptr tree_ = IupGetHandle("tree")

''  IupSetAttribute(tree, "FONT","COURIER_NORMAL")
  
	IupSetAttribute(tree_, "NAME","Figures")
  IupSetAttribute(tree_, "ADDBRANCH","3D")
  IupSetAttribute(tree_, "ADDBRANCH","2D")
  IupSetAttribute(tree_, "ADDLEAF","test")
  IupSetAttribute(tree_, "ADDBRANCH1","parallelogram")
  IupSetAttribute(tree_, "ADDLEAF2","diamond")
  IupSetAttribute(tree_, "ADDLEAF2","square")
  IupSetAttribute(tree_, "ADDBRANCH1","triangle")
  IupSetAttribute(tree_, "ADDLEAF2","scalenus")
  IupSetAttribute(tree_, "ADDLEAF2","isoceles")
  IupSetAttribute(tree_, "ADDLEAF2","equilateral")
  IupSetAttribute(tree_, "VALUE","6")
  IupSetAttribute(tree_, "CTRL","YES")
  IupSetAttribute(tree_, "SHIFT","YES")
  IupSetAttribute(tree_, "ADDEXPANDED", "NO")
  IupSetAttribute(tree_, "SHOWDRAGDROP", "YES")
  IupSetAttribute(tree_, "SHOWRENAME", "YES")	

  IupSetAttribute(tree_, "REDRAW", "YES")

End Sub

/' Main program '/
Sub main()

  Dim As Ihandle Ptr dlg
  
  IupOpen(NULL,NULL)                            /' IUP initialization '/
  IupControlsOpen()                    /' Initializes the controls library '/

  init_tree()                          /' Initializes IupTree '/
  init_dlg()                           /' Initializes the dialog '/
  dlg = IupGetHandle("dlg")            /' Retrieves the dialog handle '/
	IupShow(dlg) 													/' Displays the dialog '/
  init_tree_atributes()                /' Initializes attributes, can be done here or anywhere '/
  IupSetFocus(IupGetHandle("tree"))
	IupMainLoop()                         /' Main loop '/
  IupClose()                            /' Ends IUP '/

  
	
End Sub

main()
